@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Add temporaries, such as for clean nodes
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -20,13 +20,13 @@
 // V3Clean's Transformations:
 //
 // Each module:
-//	For each math operator, if it requires a clean operand,
-//	and the operand is dirty, insert a CLEAN node.
-//	Resize operands to C++ 32/64/wide types.
-//	Copy all width() values to widthMin() so RANGE, etc can still see orig widths
+//      For each math operator, if it requires a clean operand,
+//      and the operand is dirty, insert a CLEAN node.
+//      Resize operands to C++ 32/64/wide types.
+//      Copy all width() values to widthMin() so RANGE, etc can still see orig widths
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -44,12 +44,12 @@ class CleanVisitor : public AstNVisitor {
 private:
     // NODE STATE
     // Entire netlist:
-    //  AstNode::user()		-> CleanState.  For this node, 0==UNKNOWN
-    //  AstNode::user2()	-> bool.  True indicates widthMin has been propagated
-    //  AstNodeDType::user3()	-> AstNodeDType*.  Alternative node with C size
-    AstUser1InUse	m_inuser1;
-    AstUser2InUse	m_inuser2;
-    AstUser3InUse	m_inuser3;
+    //  AstNode::user()         -> CleanState.  For this node, 0==UNKNOWN
+    //  AstNode::user2()        -> bool.  True indicates widthMin has been propagated
+    //  AstNodeDType::user3()   -> AstNodeDType*.  Alternative node with C size
+    AstUser1InUse       m_inuser1;
+    AstUser2InUse       m_inuser2;
+    AstUser3InUse       m_inuser3;
 
     // TYPES
     enum CleanState { CS_UNKNOWN, CS_CLEAN, CS_DIRTY };
@@ -62,153 +62,157 @@ private:
 
     // Width resetting
     int  cppWidth(AstNode* nodep) {
-	if (nodep->width()<=VL_WORDSIZE) return VL_WORDSIZE;
-	else if (nodep->width()<=VL_QUADSIZE) return VL_QUADSIZE;
-	else return nodep->widthWords()*VL_WORDSIZE;
+        if (nodep->width()<=VL_WORDSIZE) return VL_WORDSIZE;
+        else if (nodep->width()<=VL_QUADSIZE) return VL_QUADSIZE;
+        else return nodep->widthWords()*VL_WORDSIZE;
     }
     void setCppWidth(AstNode* nodep) {
-	nodep->user2(true);  // Don't resize it again
-	AstNodeDType* old_dtypep = nodep->dtypep();
-	int width=cppWidth(nodep);  // widthMin is unchanged
-	if (old_dtypep->width() != width) {
-	    // Since any given dtype's cppWidth() is the same, we can just
-	    // remember one convertion for each, and reuse it
+        nodep->user2(true);  // Don't resize it again
+        AstNodeDType* old_dtypep = nodep->dtypep();
+        int width = cppWidth(nodep);  // widthMin is unchanged
+        if (old_dtypep->width() != width) {
+            // Since any given dtype's cppWidth() is the same, we can just
+            // remember one conversion for each, and reuse it
             if (AstNodeDType* new_dtypep = VN_CAST(old_dtypep->user3p(), NodeDType)) {
-		nodep->dtypep(new_dtypep);
-	    } else {
-		nodep->dtypeChgWidth(width, nodep->widthMin());
-		AstNodeDType* new_dtypep2 = nodep->dtypep();
-		if (new_dtypep2 == old_dtypep) nodep->v3fatalSrc("Dtype didn't change when width changed");
-		old_dtypep->user3p(new_dtypep2);  // Remember for next time
-	    }
-	}
+                nodep->dtypep(new_dtypep);
+            } else {
+                nodep->dtypeChgWidth(width, nodep->widthMin());
+                AstNodeDType* new_dtypep2 = nodep->dtypep();
+                UASSERT_OBJ(new_dtypep2 != old_dtypep, nodep,
+                            "Dtype didn't change when width changed");
+                old_dtypep->user3p(new_dtypep2);  // Remember for next time
+            }
+        }
     }
     void computeCppWidth(AstNode* nodep) {
-	if (!nodep->user2() && nodep->hasDType()) {
+        if (!nodep->user2() && nodep->hasDType()) {
             if (VN_IS(nodep, Var) || VN_IS(nodep, NodeDType)  // Don't want to change variable widths!
-                || VN_IS(nodep->dtypep()->skipRefp(), UnpackArrayDType)) {  // Or arrays
-	    } else {
-		setCppWidth(nodep);
-	    }
-	}
+                || VN_IS(nodep->dtypep()->skipRefp(), AssocArrayDType)  // Or arrays
+                || VN_IS(nodep->dtypep()->skipRefp(), QueueDType)
+                || VN_IS(nodep->dtypep()->skipRefp(), UnpackArrayDType)
+                || VN_IS(nodep->dtypep()->skipRefp(), VoidDType)) {
+            } else {
+                setCppWidth(nodep);
+            }
+        }
     }
 
     // Store the clean state in the userp on each node
     void setCleanState(AstNode* nodep, CleanState clean) {
-	nodep->user1(clean);
+        nodep->user1(clean);
     }
     CleanState getCleanState(AstNode* nodep) {
         return static_cast<CleanState>(nodep->user1());
     }
     bool isClean(AstNode* nodep) {
-	CleanState clstate = getCleanState(nodep);
-	if (clstate==CS_CLEAN) return true;
-	if (clstate==CS_DIRTY) return false;
-	nodep->v3fatalSrc("Unknown clean state on node: "+nodep->prettyTypeName());
-	return false;
+        CleanState clstate = getCleanState(nodep);
+        if (clstate==CS_CLEAN) return true;
+        if (clstate==CS_DIRTY) return false;
+        nodep->v3fatalSrc("Unknown clean state on node: "+nodep->prettyTypeName());
+        return false;
     }
     void setClean(AstNode* nodep, bool isClean) {
         computeCppWidth(nodep);  // Just to be sure it's in widthMin
-	bool wholeUint = ((nodep->widthMin() % VL_WORDSIZE) == 0);  //32,64,...
-	setCleanState(nodep, ((isClean||wholeUint) ? CS_CLEAN:CS_DIRTY));
+        bool wholeUint = ((nodep->widthMin() % VL_WORDSIZE) == 0);  // 32,64,...
+        setCleanState(nodep, ((isClean||wholeUint) ? CS_CLEAN:CS_DIRTY));
     }
 
     // Operate on nodes
     void insertClean(AstNode* nodep) {  // We'll insert ABOVE passed node
-	UINFO(4,"  NeedClean "<<nodep<<endl);
-	AstNRelinker relinkHandle;
-	nodep->unlinkFrBack(&relinkHandle);
-	//
-	computeCppWidth(nodep);
-	V3Number mask (nodep->fileline(), cppWidth(nodep));
-	mask.setMask(nodep->widthMin());
+        UINFO(4,"  NeedClean "<<nodep<<endl);
+        AstNRelinker relinkHandle;
+        nodep->unlinkFrBack(&relinkHandle);
+        //
+        computeCppWidth(nodep);
+        V3Number mask (nodep, cppWidth(nodep));
+        mask.setMask(nodep->widthMin());
         AstNode* cleanp = new AstAnd(nodep->fileline(),
                                      new AstConst(nodep->fileline(), mask),
                                      nodep);
-	cleanp->dtypeFrom(nodep);  // Otherwise the AND normally picks LHS
-	relinkHandle.relink(cleanp);
+        cleanp->dtypeFrom(nodep);  // Otherwise the AND normally picks LHS
+        relinkHandle.relink(cleanp);
     }
     void insureClean(AstNode* nodep) {
-	computeCppWidth(nodep);
-	if (!isClean(nodep)) insertClean(nodep);
+        computeCppWidth(nodep);
+        if (!isClean(nodep)) insertClean(nodep);
     }
     void insureCleanAndNext(AstNode* nodep) {
-	// Editing list, careful looping!
-	for (AstNode* exprp = nodep; exprp; ) {
-	    AstNode* nextp = exprp->nextp();
-	    insureClean(exprp);
-	    exprp = nextp;
-	}
+        // Editing list, careful looping!
+        for (AstNode* exprp = nodep; exprp; ) {
+            AstNode* nextp = exprp->nextp();
+            insureClean(exprp);
+            exprp = nextp;
+        }
     }
 
     // Base type handling methods
     void operandBiop(AstNodeBiop* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
-	if (nodep->cleanLhs()) {
-	    insureClean(nodep->lhsp());
-	}
-	if (nodep->cleanRhs()) {
-	    insureClean(nodep->rhsp());
-	}
-	//no setClean.. must do it in each user routine.
+        computeCppWidth(nodep);
+        if (nodep->cleanLhs()) {
+            insureClean(nodep->lhsp());
+        }
+        if (nodep->cleanRhs()) {
+            insureClean(nodep->rhsp());
+        }
+        //no setClean.. must do it in each user routine.
     }
     void operandTriop(AstNodeTriop* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
-	if (nodep->cleanLhs()) {
-	    insureClean(nodep->lhsp());
-	}
-	if (nodep->cleanRhs()) {
-	    insureClean(nodep->rhsp());
-	}
-	if (nodep->cleanThs()) {
-	    insureClean(nodep->thsp());
-	}
-	//no setClean.. must do it in each user routine.
+        computeCppWidth(nodep);
+        if (nodep->cleanLhs()) {
+            insureClean(nodep->lhsp());
+        }
+        if (nodep->cleanRhs()) {
+            insureClean(nodep->rhsp());
+        }
+        if (nodep->cleanThs()) {
+            insureClean(nodep->thsp());
+        }
+        //no setClean.. must do it in each user routine.
     }
 
     // VISITORS
     virtual void visit(AstNodeModule* nodep) {
-	m_modp = nodep;
+        m_modp = nodep;
         iterateChildren(nodep);
-	m_modp = NULL;
+        m_modp = NULL;
     }
     virtual void visit(AstNodeUniop* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
-	if (nodep->cleanLhs()) {
-	    insureClean(nodep->lhsp());
-	}
+        computeCppWidth(nodep);
+        if (nodep->cleanLhs()) {
+            insureClean(nodep->lhsp());
+        }
         setClean(nodep, nodep->cleanOut());
     }
     virtual void visit(AstNodeBiop* nodep) {
-	operandBiop(nodep);
+        operandBiop(nodep);
         setClean(nodep, nodep->cleanOut());
     }
     virtual void visit(AstAnd* nodep) {
-	operandBiop(nodep);
+        operandBiop(nodep);
         setClean(nodep, isClean(nodep->lhsp()) || isClean(nodep->rhsp()));
     }
     virtual void visit(AstXor* nodep) {
-	operandBiop(nodep);
+        operandBiop(nodep);
         setClean(nodep, isClean(nodep->lhsp()) && isClean(nodep->rhsp()));
     }
     virtual void visit(AstOr* nodep) {
-	operandBiop(nodep);
+        operandBiop(nodep);
         setClean(nodep, isClean(nodep->lhsp()) && isClean(nodep->rhsp()));
     }
     virtual void visit(AstNodeMath* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
+        computeCppWidth(nodep);
         setClean(nodep, nodep->cleanOut());
     }
     virtual void visit(AstNodeAssign* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
-	if (nodep->cleanRhs()) {
-	    insureClean(nodep->rhsp());
-	}
+        computeCppWidth(nodep);
+        if (nodep->cleanRhs()) {
+            insureClean(nodep->rhsp());
+        }
     }
     virtual void visit(AstText* nodep) {
         setClean(nodep, true);
@@ -217,17 +221,17 @@ private:
         setClean(nodep, true);
     }
     virtual void visit(AstSel* nodep) {
-	operandTriop(nodep);
+        operandTriop(nodep);
         setClean(nodep, nodep->cleanOut());
     }
     virtual void visit(AstUCFunc* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
+        computeCppWidth(nodep);
         setClean(nodep, false);
-	// We always clean, as we don't trust those pesky users.
+        // We always clean, as we don't trust those pesky users.
         if (!VN_IS(nodep->backp(), And)) {
-	    insertClean(nodep);
-	}
+            insertClean(nodep);
+        }
         insureCleanAndNext(nodep->bodysp());
     }
     virtual void visit(AstTraceDecl* nodep) {
@@ -239,32 +243,32 @@ private:
         insureCleanAndNext(nodep->valuep());
     }
     virtual void visit(AstTypedef* nodep) {
-	// No cleaning, or would loose pointer to enum
+        // No cleaning, or would loose pointer to enum
         iterateChildren(nodep);
     }
     virtual void visit(AstParamTypeDType* nodep) {
-	// No cleaning, or would loose pointer to enum
+        // No cleaning, or would loose pointer to enum
         iterateChildren(nodep);
     }
 
     // Control flow operators
     virtual void visit(AstNodeCond* nodep) {
         iterateChildren(nodep);
-	insureClean(nodep->condp());
-	setClean(nodep, isClean(nodep->expr1p()) && isClean(nodep->expr2p()));
+        insureClean(nodep->condp());
+        setClean(nodep, isClean(nodep->expr1p()) && isClean(nodep->expr2p()));
     }
     virtual void visit(AstWhile* nodep) {
         iterateChildren(nodep);
-	insureClean(nodep->condp());
+        insureClean(nodep->condp());
     }
     virtual void visit(AstNodeIf* nodep) {
         iterateChildren(nodep);
-	insureClean(nodep->condp());
+        insureClean(nodep->condp());
     }
     virtual void visit(AstSFormatF* nodep) {
         iterateChildren(nodep);
         insureCleanAndNext(nodep->exprsp());
-	setClean(nodep, true);  // generates a string, so not relevant
+        setClean(nodep, true);  // generates a string, so not relevant
     }
     virtual void visit(AstUCStmt* nodep) {
         iterateChildren(nodep);
@@ -275,16 +279,25 @@ private:
         insureCleanAndNext(nodep->argsp());
         setClean(nodep, true);
     }
+    virtual void visit(AstCMethodCall* nodep) {
+        iterateChildren(nodep);
+        insureCleanAndNext(nodep->pinsp());
+        setClean(nodep, true);
+    }
+    virtual void visit(AstIntfRef* nodep) {
+        iterateChildren(nodep);
+        setClean(nodep, true);  // generates a string, so not relevant
+    }
 
     //--------------------
     // Default: Just iterate
     virtual void visit(AstNode* nodep) {
         iterateChildren(nodep);
-	computeCppWidth(nodep);
+        computeCppWidth(nodep);
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit CleanVisitor(AstNetlist* nodep) {
         m_modp = NULL;
         iterate(nodep);
