@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Combine common code into functions
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -68,7 +68,7 @@ protected:
     // Note this is disabled, it still needed work
     // Also repair it for DPI functions; when make __common need to insure proper
     // flags get inherited from the old to new AstCFunc, and that AstText doesn't
-    // get split between functions causing the text to have a danginling reference.
+    // get split between functions causing the text to have a dangling reference.
     bool statementCombine() { return false; }  // duplicateFunctionCombine();
 };
 
@@ -96,7 +96,8 @@ public:
             AstCCall* callp = eqit->second;
             if (!callp->user3()) {  // !already done
                 UINFO(4, "     Called "<<callp<<endl);
-                if (callp->funcp() != oldfuncp) callp->v3fatalSrc("Call list broken, points to call w/different func");
+                UASSERT_OBJ(callp->funcp() == oldfuncp, callp,
+                            "Call list broken, points to call w/different func");
                 if (newfuncp) {
                     AstCCall* newp = new AstCCall(callp, newfuncp);
                     // Special new AstCCall form above transfers children of callp to newfuncp
@@ -188,7 +189,7 @@ private:
 
     // STATE
     typedef enum {STATE_IDLE, STATE_HASH, STATE_DUP} CombineState;
-    V3Double0           m_statCombs;    // Statistic tracking
+    VDouble0            m_statCombs;    // Statistic tracking
     CombineState        m_state;        // Major state
     AstNodeModule*      m_modp;         // Current module
     AstCFunc*           m_funcp;        // Current function
@@ -236,7 +237,7 @@ private:
             V3Hash hashval = it->first;
             AstNode* node1p = it->second;
             if (!VN_IS(node1p, CFunc)) continue;
-            if (hashval.isIllegal()) node1p->v3fatalSrc("Illegal (unhashed) nodes");
+            UASSERT_OBJ(!hashval.isIllegal(), node1p, "Illegal (unhashed) nodes");
             for (V3Hashed::iterator eqit = it; eqit != m_hashed.end(); ++eqit) {
                 AstNode* node2p = eqit->second;
                 if (!(eqit->first == hashval)) break;
@@ -460,7 +461,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit CombineVisitor(AstNetlist* nodep) {
         m_state = STATE_IDLE;
         m_modp = NULL;
