@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Netlist (top level) functions
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -20,7 +20,7 @@
 // COVERAGEJOIN TRANSFORMATIONS:
 //      If two COVERTOGGLEs have same VARSCOPE, combine them
 //*************************************************************************
-
+
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -50,7 +50,7 @@ private:
     // STATE
     ToggleList          m_toggleps;     // List of of all AstCoverToggle's
 
-    V3Double0           m_statToggleJoins;  // Statistic tracking
+    VDouble0            m_statToggleJoins;  // Statistic tracking
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -81,7 +81,7 @@ private:
                     // duplicated), not the covertoggle, but we need to get back to the
                     // covertoggle which is immediately above, so:
                     AstCoverToggle* removep = VN_CAST(duporigp->backp(), CoverToggle);
-                    if (!removep) nodep->v3fatalSrc("CoverageJoin duplicate of wrong type");
+                    UASSERT_OBJ(removep, nodep, "CoverageJoin duplicate of wrong type");
                     UINFO(8,"  Orig "<<nodep<<" -->> "<<nodep->incp()->declp()<<endl);
                     UINFO(8,"   dup "<<removep<<" -->> "<<removep->incp()->declp()<<endl);
                     // The CoverDecl the duplicate pointed to now needs to point to the
@@ -119,7 +119,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit CoverageJoinVisitor(AstNetlist* nodep) {
         iterate(nodep);
     }
