@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Error handling
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -17,7 +17,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 #include "V3Error.h"
 #ifndef _V3ERROR_NO_GLOBAL_
 # include "V3Ast.h"
@@ -38,6 +38,7 @@ bool V3Error::s_warnFatal = true;
 int V3Error::s_tellManual = 0;
 std::ostringstream V3Error::s_errorStr;  // Error string being formed
 V3ErrorCode V3Error::s_errorCode = V3ErrorCode::EC_FATAL;
+bool V3Error::s_errorContexted = false;
 bool V3Error::s_errorSuppressed = false;
 bool V3Error::s_describedEachWarn[V3ErrorCode::_ENUM_MAX];
 bool V3Error::s_describedWarnings = false;
@@ -72,8 +73,7 @@ void V3Error::init() {
         s_describedEachWarn[i] = false;
         s_pretendError[i] = V3ErrorCode(i).pretendError();
     }
-
-    if (string(V3ErrorCode(V3ErrorCode::_ENUM_MAX).ascii()) != " MAX") {
+    if (VL_UNCOVERABLE(string(V3ErrorCode(V3ErrorCode::_ENUM_MAX).ascii()) != " MAX")) {
         v3fatalSrc("Enum table in V3ErrorCode::EC_ascii() is munged");
     }
 }
@@ -141,7 +141,7 @@ void V3Error::vlAbort() {
         std::cerr<<msgPrefix()<<"Aborting since under --debug"<<endl;
         abort();
     } else {
-        exit(10);
+        exit(1);
     }
 }
 
@@ -161,7 +161,7 @@ string V3Error::warnMore() {
     return string(msgPrefix().size(), ' ');
 }
 
-void V3Error::v3errorEnd(std::ostringstream& sstr) {
+void V3Error::v3errorEnd(std::ostringstream& sstr, const string& locationStr) {
 #if defined(__COVERITY__) || defined(__cppcheck__)
     if (s_errorCode==V3ErrorCode::EC_FATAL) __coverity_panic__(x);
 #endif
@@ -170,10 +170,29 @@ void V3Error::v3errorEnd(std::ostringstream& sstr) {
         // On debug, show only non default-off warning to prevent pages of warnings
         && (!debug() || s_errorCode.defaultsOff())) return;
     string msg = msgPrefix()+sstr.str();
-    if (msg[msg.length()-1] != '\n') msg += '\n';
-    // Suppress duplicates
+    if (s_errorSuppressed) {  // If suppressed print only first line to reduce verbosity
+        string::size_type pos;
+        if ((pos = msg.find('\n')) != string::npos) {
+            msg.erase(pos, msg.length()-pos);
+            msg += "...";
+        }
+    }
+    // Trailing newline (generally not on messages) & remove dup newlines
+    {
+        msg += '\n';  // Trailing newlines generally not put on messages so add
+        string::size_type pos;
+        while ((pos = msg.find("\n\n")) != string::npos) {
+            msg.erase(pos+1, 1);
+        }
+    }
+    // Suppress duplicate messages
     if (s_messages.find(msg) != s_messages.end()) return;
     s_messages.insert(msg);
+    if (!locationStr.empty()) {
+        string locationMsg = warnMore()+locationStr+"\n";
+        size_t pos = msg.find("\n");
+        msg.insert(pos + 1, locationMsg);
+    }
     // Output
     std::cerr<<msg;
     if (!s_errorSuppressed && !(s_errorCode==V3ErrorCode::EC_INFO
@@ -209,7 +228,7 @@ void V3Error::v3errorEnd(std::ostringstream& sstr) {
             if (!inFatal) {
                 inFatal = true;
                 if (s_tellManual==1) {
-                    std::cerr<<warnMore()<<"... See the manual and http://www.veripool.org/verilator for more assistance."<<endl;
+                    std::cerr<<warnMore()<<"... See the manual and https://verilator.org for more assistance."<<endl;
                     s_tellManual = 2;
                 }
 #ifndef _V3ERROR_NO_GLOBAL_
