@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Graph optimizations
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -17,7 +17,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -40,7 +40,7 @@ DfaVertex* DfaGraph::findStart() {
          vertexp; vertexp=vertexp->verticesNextp()) {
         if (DfaVertex* vvertexp = dynamic_cast<DfaVertex*>(vertexp)) {
             if (vvertexp->start()) {
-                if (startp) vertexp->v3fatalSrc("Multiple start points in NFA graph");
+                UASSERT_OBJ(!startp, vertexp, "Multiple start points in NFA graph");
                 startp = vvertexp;
             }
         } else {
@@ -118,9 +118,8 @@ private:
                 DfaVertex* nfaStatep = static_cast<DfaVertex*>(dfaEdgep->top());
                 hash ^= hashVertex(nfaStatep);
                 if (debug()) {
-                    if (nfaStatep->user()==m_step) {
-                        nfaStatep->v3fatalSrc("DFA state points to duplicate NFA state.");
-                    }
+                    UASSERT_OBJ(nfaStatep->user() != m_step, nfaStatep,
+                                "DFA state points to duplicate NFA state.");
                     nfaStatep->user(m_step);
                 }
             }
@@ -220,19 +219,23 @@ private:
 
         // Expand the nfasWithInput list to include epsilon states
         // reachable by those on nfasWithInput
-        for (DfaStates::iterator nfaIt=nfasWithInput.begin();
-             nfaIt!=nfasWithInput.end(); ++nfaIt) {
-            DfaVertex* nfaStatep = *nfaIt;
-            // Foreach epsilon-reachable (on this nfaStatep)
-            for (V3GraphEdge* nfaEdgep = nfaStatep->outBeginp();
-                 nfaEdgep; nfaEdgep=nfaEdgep->outNextp()) {
-                DfaEdge* cNfaEdgep = static_cast<DfaEdge*>(nfaEdgep);
-                if (cNfaEdgep->epsilon()) {
-                    DfaVertex* nextStatep = static_cast<DfaVertex*>(cNfaEdgep->top());
-                    if (unseenNfaThisStep(nextStatep)) {  // Not processed?
-                        nfasWithInput.push_back(nextStatep);
-                        nextStatep->user(m_step);
-                        UINFO(9,"      Epsilon Reachable "<<nextStatep<<endl);
+        {
+            DfaStates nfasTodo = nfasWithInput;
+            nfasWithInput.clear();  // Now the completed list
+            while (!nfasTodo.empty()) {
+                DfaVertex* nfaStatep = nfasTodo.front(); nfasTodo.pop_front();
+                nfasWithInput.push_back(nfaStatep);
+                // Foreach epsilon-reachable (on this nfaStatep)
+                for (V3GraphEdge* nfaEdgep = nfaStatep->outBeginp();
+                     nfaEdgep; nfaEdgep=nfaEdgep->outNextp()) {
+                    DfaEdge* cNfaEdgep = static_cast<DfaEdge*>(nfaEdgep);
+                    if (cNfaEdgep->epsilon()) {
+                        DfaVertex* nextStatep = static_cast<DfaVertex*>(cNfaEdgep->top());
+                        if (unseenNfaThisStep(nextStatep)) {  // Not processed?
+                            nfasTodo.push_back(nextStatep);
+                            nextStatep->user(m_step);
+                            UINFO(9,"      Epsilon Reachable "<<nextStatep<<endl);
+                        }
                     }
                 }
             }
@@ -381,7 +384,11 @@ void DfaGraph::nfaToDfa() {
 class DfaGraphReduce : GraphAlg<> {
 private:
     // METHODS
+#ifdef VL_CPPCHECK
+    static int debug() { return 9; }
+#else
     static int debug() { return 0; }
+#endif
     DfaGraph* graphp() { return static_cast<DfaGraph*>(m_graphp); }
 
     bool isDead(DfaVertex* vertexp) {
@@ -519,7 +526,7 @@ void DfaGraph::dfaReduce() {
 // this so we just create a edge for each case and mark it "complemented."
 //
 // 3. Delete temp vertex (old accept/new reject) and related edges.
-// The user's old accept is now the new accept.  This is imporant as
+// The user's old accept is now the new accept.  This is important as
 // we want the virtual type of it to be intact.
 
 class DfaGraphComplement : GraphAlg<> {
