@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Hashed common code into functions
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -59,16 +59,15 @@ private:
     void nodeHashIterate(AstNode* nodep) {
         V3Hash thisHash;
         if (!m_cacheInUser4 || !nodep->user4()) {
-            if (VN_IS(nodep->backp(), CFunc)
-                && !(VN_IS(nodep, NodeStmt) || VN_IS(nodep, CFunc))) {
-                nodep->v3fatalSrc("Node "<<nodep->prettyTypeName()<<" in statement position but not marked stmt (node under function)");
-            }
+            UASSERT_OBJ(!(VN_IS(nodep->backp(), CFunc)
+                          && !(VN_IS(nodep, NodeStmt) || VN_IS(nodep, CFunc))), nodep,
+                        "Node "<<nodep->prettyTypeName()
+                        <<" in statement position but not marked stmt (node under function)");
             V3Hash oldHash = m_lowerHash;
             {
                 m_lowerHash = nodep->sameHash();
-                if (m_lowerHash.isIllegal()) {
-                    nodep->v3fatalSrc("sameHash function undefined (returns 0) for node under CFunc.");
-                }
+                UASSERT_OBJ(!m_lowerHash.isIllegal(), nodep,
+                            "sameHash function undefined (returns 0) for node under CFunc.");
                 // For identical nodes, the type should be the same thus
                 // dtypep should be the same too
                 m_lowerHash = V3Hash(m_lowerHash, V3Hash(nodep->type()<<6,
@@ -97,7 +96,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit HashedVisitor(AstNode* nodep) {
         m_cacheInUser4 = true;
         nodeHashIterate(nodep);
@@ -132,8 +131,8 @@ void V3Hashed::hash(AstNode* nodep) {
 }
 
 bool V3Hashed::sameNodes(AstNode* node1p, AstNode* node2p) {
-    if (!node1p->user4p()) node1p->v3fatalSrc("Called isIdentical on non-hashed nodes");
-    if (!node2p->user4p()) node2p->v3fatalSrc("Called isIdentical on non-hashed nodes");
+    UASSERT_OBJ(node1p->user4p(), node1p, "Called isIdentical on non-hashed nodes");
+    UASSERT_OBJ(node2p->user4p(), node2p, "Called isIdentical on non-hashed nodes");
     return (node1p->user4p() == node2p->user4p()  // Same hash
             && node1p->sameTree(node2p));
 }
@@ -141,11 +140,18 @@ bool V3Hashed::sameNodes(AstNode* node1p, AstNode* node2p) {
 void V3Hashed::erase(iterator it) {
     AstNode* nodep = iteratorNodep(it);
     UINFO(8,"   erase "<<nodep<<endl);
-    if (!nodep->user4p()) nodep->v3fatalSrc("Called removeNode on non-hashed node");
+    UASSERT_OBJ(nodep->user4p(), nodep, "Called removeNode on non-hashed node");
     m_hashMmap.erase(it);
     nodep->user4p(NULL);  // So we don't allow removeNode again
 }
 
+void V3Hashed::check() {
+    for (HashMmap::iterator it = begin(); it != end(); ++it) {
+        AstNode* nodep = it->second;
+        UASSERT_OBJ(nodep->user4p(), nodep, "V3Hashed check failed, non-hashed node");
+    }
+}
+
 void V3Hashed::dumpFilePrefixed(const string& nameComment, bool tree) {
     if (v3Global.opt.dumpTree()) {
         dumpFile(v3Global.debugFilename(nameComment)+".hash", tree);
@@ -161,7 +167,7 @@ void V3Hashed::dumpFile(const string& filename, bool tree) {
     V3Hash lasthash;
     int num_in_bucket = 0;
     for (HashMmap::iterator it=begin(); 1; ++it) {
-        if (lasthash != it->first || it==end()) {
+        if (it==end() || lasthash != it->first) {
             if (it!=end()) lasthash = it->first;
             if (num_in_bucket) {
                 if (dist.find(num_in_bucket)==dist.end()) {
@@ -190,31 +196,20 @@ void V3Hashed::dumpFile(const string& filename, bool tree) {
         *logp <<"\t"<<it->second<<endl;
         // Dumping the entire tree may make nearly N^2 sized dumps,
         // because the nodes under this one may also be in the hash table!
-        if (tree) it->second->dumpTree(*logp, "\t\t");
+        if (tree) it->second->dumpTree(*logp, "    ");
     }
 }
 
-V3Hashed::iterator V3Hashed::findDuplicate(AstNode* nodep) {
+V3Hashed::iterator V3Hashed::findDuplicate(AstNode* nodep, V3HashedUserSame* checkp) {
     UINFO(8,"   findD "<<nodep<<endl);
-    if (!nodep->user4p()) nodep->v3fatalSrc("Called findDuplicate on non-hashed node");
-    std::pair<HashMmap::iterator,HashMmap::iterator> eqrange = mmap().equal_range(nodeHash(nodep));
-    for (HashMmap::iterator eqit = eqrange.first; eqit != eqrange.second; ++eqit) {
-        AstNode* node2p = eqit->second;
-        if (nodep != node2p && sameNodes(nodep, node2p)) {
-            return eqit;
-        }
-    }
-    return end();
-}
-
-V3Hashed::iterator V3Hashed::findDuplicate(AstNode* nodep, V3HashedUserCheck* checkp) {
-    UINFO(8,"   findD "<<nodep<<endl);
-    if (!nodep->user4p()) nodep->v3fatalSrc("Called findDuplicate on non-hashed node");
+    UASSERT_OBJ(nodep->user4p(), nodep, "Called findDuplicate on non-hashed node");
     std::pair<HashMmap::iterator,HashMmap::iterator> eqrange
         = mmap().equal_range(nodeHash(nodep));
     for (HashMmap::iterator eqit = eqrange.first; eqit != eqrange.second; ++eqit) {
         AstNode* node2p = eqit->second;
-        if (nodep != node2p && checkp->check(nodep, node2p) && sameNodes(nodep, node2p)) {
+        if (nodep != node2p
+            && (!checkp || checkp->isSame(nodep, node2p))
+            && sameNodes(nodep, node2p)) {
             return eqit;
         }
     }
