@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Add temporaries, such as for inline nodes
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -27,7 +27,7 @@
 //          Insert cell's module statements into the upper module
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -70,7 +70,7 @@ private:
 
     // STATE
     AstNodeModule*      m_modp;         // Current module
-    V3Double0           m_statUnsup;    // Statistic tracking
+    VDouble0            m_statUnsup;    // Statistic tracking
 
     typedef std::vector<AstNodeModule*> ModVec;
     ModVec m_allMods;  // All modules, in top-down order.
@@ -124,7 +124,7 @@ private:
         if (nodep->pragType() == AstPragmaType::INLINE_MODULE) {
             //UINFO(0,"PRAG MARK "<<m_modp<<endl);
             if (!m_modp) {
-                nodep->v3error("Inline pragma not under a module");
+                nodep->v3error("Inline pragma not under a module");  // LCOV_EXCL_LINE
             } else if (m_modp->user2() == CIL_MAYBE
                        || m_modp->user2() == CIL_NOTSOFT) {
                 m_modp->user2(CIL_USER);
@@ -132,7 +132,7 @@ private:
             nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);  // Remove so don't propagate to upper cell...
         } else if (nodep->pragType() == AstPragmaType::NO_INLINE_MODULE) {
             if (!m_modp) {
-                nodep->v3error("Inline pragma not under a module");
+                nodep->v3error("Inline pragma not under a module");  // LCOV_EXCL_LINE
             } else {
                 cantInline("Pragma NO_INLINE_MODULE", false);
             }
@@ -212,7 +212,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit InlineMarkVisitor(AstNode* nodep) {
         m_modp = NULL;
         iterate(nodep);
@@ -251,7 +251,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit InlineCollectVisitor(AstNodeModule* nodep) {  // passed OLD module, not new one
         iterate(nodep);
     }
@@ -306,9 +306,8 @@ private:
             AstConst*  exprconstp  = VN_CAST(nodep->user2p(), Const);
             AstVarRef* exprvarrefp = VN_CAST(nodep->user2p(), VarRef);
             UINFO(8,"connectto: "<<nodep->user2p()<<endl);
-            if (!exprconstp && !exprvarrefp) {
-                nodep->v3fatalSrc("Unknown interconnect type; pinReconnectSimple should have cleared up");
-            }
+            UASSERT_OBJ(exprconstp || exprvarrefp, nodep,
+                        "Unknown interconnect type; pinReconnectSimple should have cleared up");
             if (exprconstp) {
                 m_modp->addStmtp(new AstAssignW(nodep->fileline(),
                                                 new AstVarRef(nodep->fileline(), nodep, true),
@@ -318,7 +317,7 @@ private:
                 // the logic changes up and down; if we aliased, we might
                 // remove the change detection on the output variable.
                 UINFO(9,"public pin assign: "<<exprvarrefp<<endl);
-                if (nodep->isNonOutput()) nodep->v3fatalSrc("Outputs only - inputs use AssignAlias");
+                UASSERT_OBJ(!nodep->isNonOutput(), nodep, "Outputs only - inputs use AssignAlias");
                 m_modp->addStmtp(
                     new AstAssignW(nodep->fileline(),
                                    new AstVarRef(nodep->fileline(), exprvarrefp->varp(), true),
@@ -463,7 +462,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     InlineRelinkVisitor(AstNodeModule* cloneModp, AstNodeModule* oldModp, AstCell* cellp) {
         m_modp = oldModp;
         m_cellp = cellp;
@@ -496,7 +495,7 @@ private:
 
     // STATE
     AstNodeModule*      m_modp;         // Current module
-    V3Double0           m_statCells;    // Statistic tracking
+    VDouble0            m_statCells;    // Statistic tracking
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -555,12 +554,11 @@ private:
                 // delete it in later optimizations.
                 AstVar* pinOldVarp = pinp->modVarp();
                 AstVar* pinNewVarp = pinOldVarp->clonep();
-                if (!pinNewVarp) pinOldVarp->v3fatalSrc("Cloning failed");
+                UASSERT_OBJ(pinNewVarp, pinOldVarp, "Cloning failed");
 
                 AstNode* connectRefp = pinp->exprp();
-                if (!VN_IS(connectRefp, Const) && !VN_IS(connectRefp, VarRef)) {
-                    pinp->v3fatalSrc("Unknown interconnect type; pinReconnectSimple should have cleared up");
-                }
+                UASSERT_OBJ(VN_IS(connectRefp, Const) || VN_IS(connectRefp, VarRef), pinp,
+                            "Unknown interconnect type; pinReconnectSimple should have cleared up");
                 V3Inst::checkOutputShort(pinp);
 
                 // Propagate any attributes across the interconnect
@@ -605,7 +603,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit InlineVisitor(AstNode* nodep) {
         m_modp = NULL;
         iterate(nodep);
@@ -615,6 +613,109 @@ public:
     }
 };
 
+//######################################################################
+// Track interface references under the Cell they reference
+
+class InlineIntfRefVisitor : public AstNVisitor {
+private:
+    // NODE STATE
+    //   AstVar::user1p()   // AstCell which this Var points to
+
+    AstUser2InUse m_inuser2;
+
+    string m_scope;  // Scope name
+
+    // METHODS
+    VL_DEBUG_FUNC;  // Declare debug()
+
+    // VISITORS
+    virtual void visit(AstNetlist* nodep) {
+        iterateChildren(nodep);
+    }
+
+    virtual void visit(AstModule* nodep) {
+        if (nodep->isTop()) {
+            iterateChildren(nodep);
+        }
+    }
+
+    virtual void visit(AstCell* nodep) {
+        string oldScope = m_scope;
+        if (m_scope.empty()) {
+            m_scope = nodep->name();
+        } else {
+            m_scope += "__DOT__" + nodep->name();
+        }
+
+        if (AstModule* modp = VN_CAST(nodep->modp(), Module)) {
+            // Pass Cell pointers down to the next module
+            for (AstPin* pinp = nodep->pinsp(); pinp; pinp=VN_CAST(pinp->nextp(), Pin)) {
+                AstVar* varp = pinp->modVarp();
+                AstVarRef* varrefp = VN_CAST(pinp->exprp(), VarRef);
+                if (!varrefp) continue;
+                AstVar* fromVarp = varrefp->varp();
+                AstIfaceRefDType* irdtp = VN_CAST(fromVarp->dtypep(), IfaceRefDType);
+                if (!irdtp) continue;
+
+                AstCell* cellp;
+                if ((cellp = VN_CAST(fromVarp->user1p(), Cell))
+                    || (cellp = irdtp->cellp())) {
+                    varp->user1p(cellp);
+                    string alias = m_scope + "__DOT__" + pinp->name();
+                    cellp->addIntfRefp(new AstIntfRef(pinp->fileline(), alias));
+                }
+            }
+
+            iterateChildren(modp);
+        } else if (VN_IS(nodep->modp(), Iface)) {
+            nodep->addIntfRefp(new AstIntfRef(nodep->fileline(), m_scope));
+            // No need to iterate on interface cells
+        }
+
+        m_scope = oldScope;
+    }
+
+    virtual void visit(AstAssignVarScope* nodep) {
+        // Reference
+        AstVarRef* reflp = VN_CAST(nodep->lhsp(), VarRef);
+        // What the reference refers to
+        AstVarRef* refrp = VN_CAST(nodep->rhsp(), VarRef);
+        if (!(reflp && refrp)) return;
+
+        AstVar* varlp = reflp->varp();
+        AstVar* varrp = refrp->varp();
+        if (!(varlp && varrp)) return;
+
+        AstCell* cellp = VN_CAST(varrp->user1p(), Cell);
+        if (!cellp) {
+            AstIfaceRefDType* irdtp = VN_CAST(varrp->dtypep(), IfaceRefDType);
+            if (!irdtp) return;
+
+            cellp = irdtp->cellp();
+        }
+        if (!cellp) return;
+        string alias;
+        if (!m_scope.empty()) alias = m_scope + "__DOT__";
+        alias += varlp->name();
+        cellp->addIntfRefp(new AstIntfRef(varlp->fileline(), alias));
+    }
+
+    //--------------------
+    virtual void visit(AstNodeMath*) {}  // Accelerate
+    virtual void visit(AstNodeStmt*) {}  // Accelerate
+    virtual void visit(AstNode* nodep) {
+        iterateChildren(nodep);
+    }
+
+public:
+    // CONSTRUCTORS
+    explicit InlineIntfRefVisitor(AstNode* nodep) {
+        iterate(nodep);
+    }
+    virtual ~InlineIntfRefVisitor() {
+    }
+};
+
 //######################################################################
 // Inline class functions
 
@@ -639,5 +740,8 @@ void V3Inline::inlineAll(AstNetlist* nodep) {
             modp->unlinkFrBack()->deleteTree(); VL_DANGLING(modp);
         }
     }
+    {
+        InlineIntfRefVisitor crvisitor (nodep);
+    }
     V3Global::dumpCheckGlobalTree("inline", 0, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
 }
