@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Resolve module/signal name references
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -26,7 +26,7 @@
 //              Read module if needed
 //              Link to module that instantiates it
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -79,9 +79,9 @@ public:
 void LinkCellsGraph::loopsMessageCb(V3GraphVertex* vertexp) {
     if (LinkCellsVertex* vvertexp = dynamic_cast<LinkCellsVertex*>(vertexp)) {
         vvertexp->modp()->v3error("Unsupported: Recursive multiple modules (module instantiates something leading back to itself): "
-                                  <<vvertexp->modp()->prettyName()<<endl
+                                  <<vvertexp->modp()->prettyNameQ()<<endl
                                   <<V3Error::warnMore()
-                                  <<"... Note self-recursion (module instantiating itself directly) is supported.");
+                                  <<"... note: self-recursion (module instantiating itself directly) is supported.");
         V3Error::abortIfErrors();
     } else {  // Everything should match above, but...
         v3fatalSrc("Recursive instantiations");
@@ -105,7 +105,7 @@ private:
     AstUser2InUse       m_inuser2;
 
     // STATE
-    V3InFilter*         m_filterp;      // Parser filter
+    VInFilter*          m_filterp;      // Parser filter
     V3ParseSym*         m_parseSymp;    // Parser symbol table
 
     // Below state needs to be preserved between each module call.
@@ -141,7 +141,8 @@ private:
             // We'll throw the error when we know the module will really be needed.
             string prettyName = AstNode::prettyName(modName);
             V3Parse parser (v3Global.rootp(), m_filterp, m_parseSymp);
-            parser.parseFile(nodep->fileline(), prettyName, false, "");
+            // true below -> other simulators treat modules in link-found files as library cells
+            parser.parseFile(nodep->fileline(), prettyName, true, "");
             V3Error::abortIfErrors();
             // We've read new modules, grab new pointers to their names
             readModNames();
@@ -150,7 +151,7 @@ private:
             if (!modp) {
                 // This shouldn't throw a message as parseFile will create
                 // a AstNotFoundModule for us
-                nodep->v3error("Can't resolve module reference: "<<prettyName);
+                nodep->v3error("Can't resolve module reference: '"<<prettyName<<"'");
             }
         }
         return modp;
@@ -203,7 +204,7 @@ private:
                 m_declfnWarned.insert(nodep->fileline()->filename());
                 nodep->v3warn(DECLFILENAME, "Filename '"<<nodep->fileline()->filebasenameNoExt()
                               <<"' does not match "<<nodep->typeName()
-                              <<" name: "<<nodep->prettyName());
+                              <<" name: "<<nodep->prettyNameQ());
             }
         }
         if (VN_IS(nodep, Iface) || VN_IS(nodep, Package)) nodep->inLibrary(true);  // Interfaces can't be at top, unless asked
@@ -240,7 +241,7 @@ private:
                 if (!nodep->cellp()) nodep->ifacep(VN_CAST(modp, Iface));
             } else if (VN_IS(modp, NotFoundModule)) {  // Will error out later
             } else {
-                nodep->v3error("Non-interface used as an interface: "<<nodep->prettyName());
+                nodep->v3error("Non-interface used as an interface: "<<nodep->prettyNameQ());
             }
         }
         // Note cannot do modport resolution here; modports are allowed underneath generates
@@ -249,7 +250,7 @@ private:
     virtual void visit(AstPackageImport* nodep) {
         // Package Import: We need to do the package before the use of a package
         iterateChildren(nodep);
-        if (!nodep->packagep()) nodep->v3fatalSrc("Unlinked package");  // Parser should set packagep
+        UASSERT_OBJ(nodep->packagep(), nodep, "Unlinked package");  // Parser should set packagep
         new V3GraphEdge(&m_graph, vertex(m_modp), vertex(nodep->packagep()), 1, false);
     }
 
@@ -373,11 +374,11 @@ private:
                 if (!pinp->exprp()) {
                     if (pinp->name().substr(0, 11) == "__pinNumber") {
                         pinp->v3warn(PINNOCONNECT, "Cell pin is not connected: "
-                                     <<pinp->prettyName());
+                                     <<pinp->prettyNameQ());
                     } else {
                         pinp->v3warn(PINCONNECTEMPTY,
                                      "Cell pin connected by name with empty reference: "
-                                     <<pinp->prettyName());
+                                     <<pinp->prettyNameQ());
                     }
                 }
                 if (ports.find(pinp->name()) == ports.end()) {
@@ -396,13 +397,13 @@ private:
                             // Create any not already connected
                             AstPin* newp = new AstPin(nodep->fileline(), 0, portp->name(),
                                                       new AstParseRef(nodep->fileline(),
-                                                                      AstParseRefExp::PX_TEXT,
+                                                                      VParseRefExp::PX_TEXT,
                                                                       portp->name(), NULL, NULL));
                             newp->svImplicit(true);
                             nodep->addPinsp(newp);
                         } else {  // warn on the CELL that needs it, not the port
                             nodep->v3warn(PINMISSING, "Cell has missing pin: "
-                                          <<portp->prettyName());
+                                          <<portp->prettyNameQ());
                             AstPin* newp = new AstPin(nodep->fileline(), 0, portp->name(), NULL);
                             nodep->addPinsp(newp);
                         }
@@ -463,8 +464,10 @@ private:
                 if (!(foundp->fileline()->warnIsOff(V3ErrorCode::MODDUP)
                       || nodep->fileline()->warnIsOff(V3ErrorCode::MODDUP))) {
                     nodep->v3warn(MODDUP, "Duplicate declaration of module: "
-                                  <<nodep->prettyName()<<endl
-                                  <<foundp->warnMore()<<"... Location of original declaration");
+                                  <<nodep->prettyNameQ()<<endl
+                                  <<nodep->warnContextPrimary()<<endl
+                                  <<foundp->warnOther()<<"... Location of original declaration"<<endl
+                                  <<foundp->warnContextSecondary());
                 }
                 nodep->unlinkFrBack();
                 pushDeletep(nodep); VL_DANGLING(nodep);
@@ -476,8 +479,8 @@ private:
     }
 
 public:
-    // CONSTUCTORS
-    LinkCellsVisitor(AstNetlist* nodep, V3InFilter* filterp, V3ParseSym* parseSymp)
+    // CONSTRUCTORS
+    LinkCellsVisitor(AstNetlist* nodep, VInFilter* filterp, V3ParseSym* parseSymp)
         : m_mods(nodep) {
         m_filterp = filterp;
         m_parseSymp = parseSymp;
@@ -492,7 +495,7 @@ public:
 //######################################################################
 // Link class functions
 
-void V3LinkCells::link(AstNetlist* nodep, V3InFilter* filterp, V3ParseSym* parseSymp) {
+void V3LinkCells::link(AstNetlist* nodep, VInFilter* filterp, V3ParseSym* parseSymp) {
     UINFO(4,__FUNCTION__<<": "<<endl);
     LinkCellsVisitor visitor (nodep, filterp, parseSymp);
 }
