@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Replace return/continue with jumps
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -25,7 +25,7 @@
 //      FOR -> WHILEs
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -77,16 +77,14 @@ private:
             nodep->v3fatalSrc("Unknown jump point for break/disable/continue");
             return NULL;
         }
-        // Skip over variables as we'll just move them in a momement
+        // Skip over variables as we'll just move them in a moment
         // Also this would otherwise prevent us from using a label twice
         // see t_func_return test.
         while (underp && VN_IS(underp, Var)) underp = underp->nextp();
-        if (underp) UINFO(5,"  Underpoint is "<<underp<<endl);
+        UASSERT_OBJ(underp, nodep, "Break/disable/continue not under expected statement");
+        UINFO(5,"  Underpoint is "<<underp<<endl);
 
-        if (!underp) {
-            nodep->v3fatalSrc("Break/disable/continue not under expected statement");
-            return NULL;
-        } else if (VN_IS(underp, JumpLabel)) {
+        if (VN_IS(underp, JumpLabel)) {
             return VN_CAST(underp, JumpLabel);
         } else {  // Move underp stuff to be under a new label
             AstJumpLabel* labelp = new AstJumpLabel(nodep->fileline(), NULL);
@@ -232,7 +230,7 @@ private:
         }
         //if (debug()>=9) { UINFO(0,"\n"); beginp->dumpTree(cout, "  labeli: "); }
         if (!beginp) { nodep->v3error("disable isn't underneath a begin with name: "
-                                      <<nodep->prettyName()); }
+                                      <<nodep->prettyNameQ()); }
         else {
             // Jump to the end of the named begin
             AstJumpLabel* labelp = findAddLabel(beginp, false);
@@ -250,7 +248,7 @@ private:
         iterateChildren(nodep);
     }
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit LinkJumpVisitor(AstNetlist* nodep) {
         m_modp = NULL;
         m_ftaskp = NULL;
