@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Netlist (top level) functions
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -25,7 +25,7 @@
 //         V3ParseLex           Wrapper that includes lex output
 //           V3Lexer.yy.cpp     Flex output
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -36,6 +36,7 @@
 #include "V3File.h"
 #include "V3ParseImp.h"
 #include "V3PreShell.h"
+#include "V3LanguageWords.h"
 
 #include <cstdarg>
 #include <fstream>
@@ -74,8 +75,16 @@ V3ParseImp::~V3ParseImp() {
 
 void V3ParseImp::ppline(const char* textp) {
     // Handle `line directive
+    FileLine* prevFl = copyOrSameFileLine();
     int enterExit;
     fileline()->lineDirective(textp, enterExit/*ref*/);
+    if (enterExit == 1) {  // Enter
+        fileline()->parent(prevFl);
+    } else if (enterExit == 2) {  // Exit
+        FileLine* upFl = fileline()->parent();
+        if (upFl) upFl = upFl->parent();
+        if (upFl) fileline()->parent(upFl);
+    }
 }
 
 void V3ParseImp::verilatorCmtLintSave() {
@@ -124,6 +133,21 @@ void V3ParseImp::verilatorCmtBad(const char* textp) {
     }
 }
 
+void V3ParseImp::errorPreprocDirective(const char* textp) {
+    // Find all `preprocessor spelling candidates
+    // Can't make this static as might get more defines later when read cells
+    VSpellCheck speller;
+    V3LanguageWords words;
+    for (V3LanguageWords::const_iterator it = words.begin(); it != words.end(); ++it) {
+        string ppDirective = it->first;
+        if (ppDirective[0] == '`') speller.pushCandidate(ppDirective);
+    }
+    V3PreShell::candidateDefines(&speller);
+    string suggest = speller.bestCandidateMsg(textp);
+    fileline()->v3error("Define or directive not defined: '"<<textp<<"'\n"
+                        <<(suggest.empty() ? "" : fileline()->warnMore()+suggest));
+}
+
 void V3ParseImp::tag(const char* text) {
     if (m_tagNodep) {
         string tmp = text + strlen("/*verilator tag ");
@@ -167,7 +191,7 @@ size_t V3ParseImp::ppInputToLex(char* buf, size_t max_size) {
             m_ppBuffers.push_front(remainder);  // Put back remainder for next time
             len = (max_size-got);
         }
-        strncpy(buf+got, front.c_str(), len);
+        memcpy(buf+got, front.c_str(), len);
         got += len;
     }
     if (debug()>=9) {
@@ -202,6 +226,7 @@ void V3ParseImp::parseFile(FileLine* fileline, const string& modfilename, bool i
 
     UINFO(2,__FUNCTION__<<": "<<modname<<(inLibrary?" [LIB]":"")<<endl);
     m_fileline = new FileLine(fileline);
+    m_fileline->newContent();
     m_inLibrary = inLibrary;
 
     // Preprocess into m_ppBuffer
@@ -259,7 +284,7 @@ void V3ParseImp::lexFile(const string& modname) {
 //======================================================================
 // V3Parse functions
 
-V3Parse::V3Parse(AstNetlist* rootp, V3InFilter* filterp, V3ParseSym* symp) {
+V3Parse::V3Parse(AstNetlist* rootp, VInFilter* filterp, V3ParseSym* symp) {
     m_impp = new V3ParseImp(rootp, filterp, symp);
 }
 V3Parse::~V3Parse() {
