@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Common header between parser and lex
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -17,7 +17,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 #ifndef _V3PARSEIMP_H_
 #define _V3PARSEIMP_H_ 1
 
@@ -99,7 +99,7 @@ struct V3ParseBisonYYSType {
 class V3ParseImp {
     // MEMBERS
     AstNetlist*         m_rootp;        // Root of the design
-    V3InFilter*         m_filterp;      // Reading filter
+    VInFilter*          m_filterp;      // Reading filter
     V3ParseSym*         m_symp;         // Symbol table
 
     V3Lexer*            m_lexerp;       // Current FlexLexer
@@ -148,6 +148,7 @@ public:
     void verilatorCmtLintSave();
     void verilatorCmtLintRestore();
     void verilatorCmtBad(const char* text);
+    void errorPreprocDirective(const char* textp);
     void tag(const char* text);
     void tagNodep(AstNode* nodep) { m_tagNodep = nodep; }
     AstNode* tagNodep() const { return m_tagNodep;}
@@ -159,7 +160,10 @@ public:
     int lastVerilogState() const { return m_lastVerilogState; }
     static const char* tokenName(int tok);
 
-    void ppPushText(const string& text) { m_ppBuffers.push_back(text); }
+    void ppPushText(const string& text) {
+        m_ppBuffers.push_back(text);
+        if (fileline()->contentp()) fileline()->contentp()->pushText(text);
+    }
     size_t ppInputToLex(char* buf, size_t max_size);
 
     static V3ParseImp* parsep() { return s_parsep; }
@@ -215,7 +219,7 @@ public:
 
 public:
     // CONSTRUCTORS
-    V3ParseImp(AstNetlist* rootp, V3InFilter* filterp, V3ParseSym* parserSymp)
+    V3ParseImp(AstNetlist* rootp, VInFilter* filterp, V3ParseSym* parserSymp)
         : m_rootp(rootp), m_filterp(filterp), m_symp(parserSymp) {
         m_fileline = NULL;
         m_lexerp = NULL;
