@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Preprocessing wrapper
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -17,7 +17,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -41,8 +41,8 @@ protected:
     friend class V3PreShell;
 
     static V3PreShellImp s_preImp;
-    static V3PreProc*   s_preprocp;
-    static V3InFilter*  s_filterp;
+    static V3PreProc* s_preprocp;
+    static VInFilter* s_filterp;
 
     //---------------------------------------
     // METHODS
@@ -60,11 +60,11 @@ protected:
         // Create the implementation pointer
         if (env) {}
         if (!s_preprocp) {
-            FileLine* cmdfl = new FileLine(FileLine::commandLineFilename(), 0);
+            FileLine* cmdfl = new FileLine(FileLine::commandLineFilename());
             s_preprocp = V3PreProc::createPreProc(cmdfl);
             s_preprocp->debug(debug());
             // Default defines
-            FileLine* prefl = new FileLine(FileLine::internalDefineFilename(), 0);
+            FileLine* prefl = new FileLine(FileLine::builtInFilename());
             s_preprocp->defineCmdLine(prefl, "VERILATOR", "1");  // LEAK_OK
             s_preprocp->defineCmdLine(prefl, "verilator", "1");  // LEAK_OK
             s_preprocp->defineCmdLine(prefl, "verilator3", "1");  // LEAK_OK
@@ -93,7 +93,7 @@ protected:
         }
     }
 
-    bool preproc(FileLine* fl, const string& modname, V3InFilter* filterp, V3ParseImp* parsep,
+    bool preproc(FileLine* fl, const string& modname, VInFilter* filterp, V3ParseImp* parsep,
                  const string& errmsg) {  // "" for no error
         debug(true);  // Recheck if debug on - first check was before command line passed
 
@@ -107,11 +107,11 @@ protected:
 
         // Set language standard up front
         if (!v3Global.opt.preprocOnly()) {
-            // Leting lex parse this saves us from having to specially en/decode
+            // Letting lex parse this saves us from having to specially en/decode
             // from the V3LangCode to the various Lex BEGIN states. The language
             // of this source file is updated here, in case there have been any
-            // intervening +<lang>ext+ options since it was first ecountered.
-            FileLine* modfileline = new FileLine(modfilename, 0);
+            // intervening +<lang>ext+ options since it was first encountered.
+            FileLine* modfileline = new FileLine(modfilename);
             modfileline->language(v3Global.opt.fileLanguage(modfilename));
             V3Parse::ppPushText(parsep, (string("`begin_keywords \"")
                                          +modfileline->language().ascii()+"\"\n"));
@@ -134,7 +134,7 @@ protected:
     }
 
 private:
-    string preprocOpen(FileLine* fl, V3InFilter* filterp,
+    string preprocOpen(FileLine* fl, VInFilter* filterp,
                        const string& modname, const string& lastpath,
                        const string& errmsg) {  // Error message or "" to suppress
         // Returns filename if successful
@@ -162,7 +162,7 @@ public:
 
 V3PreShellImp V3PreShellImp::s_preImp;
 V3PreProc* V3PreShellImp::s_preprocp = NULL;
-V3InFilter* V3PreShellImp::s_filterp = NULL;
+VInFilter* V3PreShellImp::s_filterp = NULL;
 
 //######################################################################
 // Perl class functions
@@ -170,7 +170,7 @@ V3InFilter* V3PreShellImp::s_filterp = NULL;
 void V3PreShell::boot(char** env) {
     V3PreShellImp::s_preImp.boot(env);
 }
-bool V3PreShell::preproc(FileLine* fl, const string& modname, V3InFilter* filterp,
+bool V3PreShell::preproc(FileLine* fl, const string& modname, VInFilter* filterp,
                          V3ParseImp* parsep, const string& errmsg) {
     return V3PreShellImp::s_preImp.preproc(fl, modname, filterp, parsep, errmsg);
 }
@@ -178,7 +178,7 @@ void V3PreShell::preprocInclude(FileLine* fl, const string& modname) {
     V3PreShellImp::s_preImp.preprocInclude(fl, modname);
 }
 void V3PreShell::defineCmdLine(const string& name, const string& value) {
-    FileLine* prefl = new FileLine(FileLine::commandLineFilename(), 0);
+    FileLine* prefl = new FileLine(FileLine::commandLineFilename());
     V3PreShellImp::s_preprocp->defineCmdLine(prefl, name, value);
 }
 void V3PreShell::undef(const string& name) {
@@ -187,3 +187,6 @@ void V3PreShell::undef(const string& name) {
 void V3PreShell::dumpDefines(std::ostream& os) {
     V3PreShellImp::s_preprocp->dumpDefines(os);
 }
+void V3PreShell::candidateDefines(VSpellCheck* spellerp) {
+    V3PreShellImp::s_preprocp->candidateDefines(spellerp);
+}
