@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Add temporaries, such as for premit nodes
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -23,11 +23,11 @@
 //      For each wide OP, make a a temporary variable with the wide value
 //      For each deep expression, assign expression to temporary.
 //
-// Each display (independant transformation; here as Premit is a good point)
+// Each display (independent transformation; here as Premit is a good point)
 //      If autoflush, insert a flush
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -349,6 +349,16 @@ private:
         }
         checkNode(nodep);
     }
+    virtual void visit(AstAssocSel* nodep) {
+        iterateAndNextNull(nodep->fromp());
+        {   // Only the 'from' is part of the assignment LHS
+            bool prevAssign = m_assignLhs;
+            m_assignLhs = false;
+            iterateAndNextNull(nodep->bitp());
+            m_assignLhs = prevAssign;
+        }
+        checkNode(nodep);
+    }
     virtual void visit(AstConst* nodep) {
         iterateChildren(nodep); checkNode(nodep);
     }
@@ -388,7 +398,8 @@ private:
         // Any strings sent to a display must be var of string data type,
         // to avoid passing a pointer to a temporary.
         for (AstNode* expp=nodep->exprsp(); expp; expp = expp->nextp()) {
-            if (expp->dtypep()->basicp()->isString()
+            if (expp->dtypep()->basicp()
+                && expp->dtypep()->basicp()->isString()
                 && !VN_IS(expp, VarRef)) {
                 createDeepTemp(expp, true);
             }
@@ -403,7 +414,7 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit PremitVisitor(AstNetlist* nodep) {
         m_modp = NULL;
         m_funcp = NULL;
