@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Break always into sensitivity block domains
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -24,7 +24,7 @@
 //                      {all blocked statements}
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -65,7 +65,7 @@ private:
 
     PackageScopeMap     m_packageScopes;        // Scopes for each package
     VarScopeMap         m_varScopes;            // Varscopes created for each scope and var
-    VarRefScopeSet      m_varRefScopes;         // Varrefs-in-scopes needing fixup when donw
+    VarRefScopeSet      m_varRefScopes;         // Varrefs-in-scopes needing fixup when done
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -77,11 +77,11 @@ private:
             AstScope* scopep = it->second;
             if (nodep->packagep()) {
                 PackageScopeMap::iterator it2 = m_packageScopes.find(nodep->packagep());
-                if (it2==m_packageScopes.end()) nodep->v3fatalSrc("Can't locate package scope");
+                UASSERT_OBJ(it2 != m_packageScopes.end(), nodep, "Can't locate package scope");
                 scopep = it2->second;
             }
             VarScopeMap::iterator it3 = m_varScopes.find(make_pair(nodep->varp(), scopep));
-            if (it3==m_varScopes.end()) nodep->v3fatalSrc("Can't locate varref scope");
+            UASSERT_OBJ(it3 != m_varScopes.end(), nodep, "Can't locate varref scope");
             AstVarScope* varscp = it3->second;
             nodep->varScopep(varscp);
         }
@@ -90,7 +90,7 @@ private:
     // VISITORS
     virtual void visit(AstNetlist* nodep) {
         AstNodeModule* modp = nodep->topModulep();
-        if (!modp) { nodep->v3error("No root module specified"); return; }
+        if (!modp) { nodep->v3error("No top level module found"); return; }
         // Operate starting at the top of the hierarchy
         m_aboveCellp = NULL;
         m_aboveScopep = NULL;
@@ -124,7 +124,7 @@ private:
                     m_aboveCellp = cellp;
                     m_aboveScopep = m_scopep;
                     AstNodeModule* modp = cellp->modp();
-                    if (!modp) cellp->v3fatalSrc("Unlinked mod");
+                    UASSERT_OBJ(modp, cellp, "Unlinked mod");
                     iterate(modp);  // Recursive call to visit(AstNodeModule)
                 }
                 // Done, restore vars
@@ -151,6 +151,9 @@ private:
 
         // ***Note m_scopep is passed back to the caller of the routine (above)
     }
+    virtual void visit(AstCellInline* nodep) {
+        nodep->scopep(m_scopep);
+    }
     virtual void visit(AstActive* nodep) {
         nodep->v3fatalSrc("Actives now made after scoping");
     }
@@ -246,12 +249,12 @@ private:
             if (m_aboveCellp && !m_aboveCellp->isTrace()) varscp->trace(false);
             nodep->user1p(varscp);
             if (v3Global.opt.isClocker(varscp->prettyName())) {
-                nodep->attrClocker(AstVarAttrClocker::CLOCKER_YES);
+                nodep->attrClocker(VVarAttrClocker::CLOCKER_YES);
             }
             if (v3Global.opt.isNoClocker(varscp->prettyName())) {
-                nodep->attrClocker(AstVarAttrClocker::CLOCKER_NO);
+                nodep->attrClocker(VVarAttrClocker::CLOCKER_NO);
             }
-            if (!m_scopep) nodep->v3fatalSrc("No scope for var");
+            UASSERT_OBJ(m_scopep, nodep, "No scope for var");
             m_varScopes.insert(make_pair(make_pair(nodep, m_scopep), varscp));
             m_scopep->addVarp(varscp);
         }
@@ -259,7 +262,7 @@ private:
     virtual void visit(AstVarRef* nodep) {
         // VarRef needs to point to VarScope
         // Make sure variable has made user1p.
-        if (!nodep->varp()) nodep->v3fatalSrc("Unlinked");
+        UASSERT_OBJ(nodep->varp(), nodep, "Unlinked");
         if (nodep->varp()->isIfaceRef()) {
             nodep->varScopep(NULL);
         } else {
@@ -295,7 +298,7 @@ private:
         iterateChildren(nodep);
     }
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit ScopeVisitor(AstNetlist* nodep) {
         m_aboveCellp = NULL;
         m_aboveScopep = NULL;
@@ -378,9 +381,9 @@ private:
         UINFO(9,"   Old pkg-taskref "<<nodep<<endl);
         if (nodep->packagep()) {
             // Point to the clone
-            if (!nodep->taskp()) nodep->v3fatalSrc("Unlinked");
+            UASSERT_OBJ(nodep->taskp(), nodep, "Unlinked");
             AstNodeFTask* newp = VN_CAST(nodep->taskp()->user2p(), NodeFTask);
-            if (!newp) nodep->v3fatalSrc("No clone for package function");
+            UASSERT_OBJ(newp, nodep, "No clone for package function");
             nodep->taskp(newp);
             UINFO(9,"   New pkg-taskref "<<nodep<<endl);
         } else {
@@ -401,7 +404,7 @@ private:
         iterateChildren(nodep);
     }
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit ScopeCleanupVisitor(AstNetlist* nodep) {
         m_scopep = NULL;
         iterate(nodep);
