@@ -9,7 +9,7 @@
 //
 // See details below
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -24,7 +24,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 #ifndef _V3SCOREBOARD_H_
 #define _V3SCOREBOARD_H_
 
@@ -125,7 +125,7 @@ public:
                 return;
             }
             // Try the previous value?
-            if (m_valIt == m_sbvmp->m_vals.begin()) {
+            if (VL_UNCOVERABLE(m_valIt == m_sbvmp->m_vals.begin())) {
                 // No more values but it's not defined to decrement an
                 // iterator past the beginning.
                 v3fatalSrc("Decremented iterator past beginning");
