@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Check for unused/undriven signals
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -26,7 +26,7 @@
 //      Report unused/undriven nets
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -183,33 +183,34 @@ public:
                 // UNDRIVEN is considered more serious - as is more likely a bug,
                 // thus undriven+unused bits get UNUSED warnings, as they're not as buggy.
                 if (!unusedMatch(nodep)) {
-                    nodep->v3warn(UNUSED, "Signal is not driven, nor used: "<<nodep->prettyName());
+                    nodep->v3warn(UNUSED, "Signal is not driven, nor used: "
+                                  <<nodep->prettyNameQ());
                     nodep->fileline()->modifyWarnOff(V3ErrorCode::UNUSED, true);  // Warn only once
                 }
             } else if (allD && !anyU) {
                 if (!unusedMatch(nodep)) {
-                    nodep->v3warn(UNUSED, "Signal is not used: "<<nodep->prettyName());
+                    nodep->v3warn(UNUSED, "Signal is not used: "<<nodep->prettyNameQ());
                     nodep->fileline()->modifyWarnOff(V3ErrorCode::UNUSED, true);  // Warn only once
                 }
             } else if (!anyD && allU) {
-                nodep->v3warn(UNDRIVEN, "Signal is not driven: "<<nodep->prettyName());
+                nodep->v3warn(UNDRIVEN, "Signal is not driven: "<<nodep->prettyNameQ());
                 nodep->fileline()->modifyWarnOff(V3ErrorCode::UNDRIVEN, true);  // Warn only once
             } else {
                 // Bits have different dispositions
                 bool setU=false; bool setD=false;
                 if (anynotDU && !unusedMatch(nodep)) {
                     nodep->v3warn(UNUSED, "Bits of signal are not driven, nor used: "
-                                  <<nodep->prettyName()
+                                  <<nodep->prettyNameQ()
                                   <<bitNames(BN_BOTH));
                     setU = true;
                 }
                 if (anyDnotU && !unusedMatch(nodep)) {
-                    nodep->v3warn(UNUSED, "Bits of signal are not used: "<<nodep->prettyName()
+                    nodep->v3warn(UNUSED, "Bits of signal are not used: "<<nodep->prettyNameQ()
                                   <<bitNames(BN_UNUSED));
                     setU = true;
                 }
                 if (anyUnotD) {
-                    nodep->v3warn(UNDRIVEN, "Bits of signal are not driven: "<<nodep->prettyName()
+                    nodep->v3warn(UNDRIVEN, "Bits of signal are not driven: "<<nodep->prettyNameQ()
                                   <<bitNames(BN_UNDRIVEN));
                     setD = true;
                 }
@@ -237,7 +238,7 @@ private:
     std::vector<UndrivenVarEntry*> m_entryps[3];  // Nodes to delete when we are finished
     bool                m_inBBox;       // In black box; mark as driven+used
     bool                m_inContAssign;  // In continuous assignment
-    bool                m_inProcAssign;  // In procedual assignment
+    bool                m_inProcAssign;  // In procedural assignment
     AstNodeFTask*       m_taskp;        // Current task
     AstAlways*          m_alwaysCombp;  // Current always if combo, otherwise NULL
 
@@ -267,7 +268,7 @@ private:
             && !VN_IS(nodep, VarXRef)  // Xrefs might point at two different instances
             && !varp->fileline()->warnIsOff(V3ErrorCode::ALWCOMBORDER)) {  // Warn only once per variable
             nodep->v3warn(ALWCOMBORDER, "Always_comb variable driven after use: "
-                          <<nodep->prettyName());
+                          <<nodep->prettyNameQ());
             varp->fileline()->modifyWarnOff(V3ErrorCode::ALWCOMBORDER, true);  // Complain just once for any usage
         }
     }
@@ -291,6 +292,9 @@ private:
                 || (m_taskp && (m_taskp->dpiImport() || m_taskp->dpiExport()))) {
                 entryp->usedWhole();
             }
+            if (nodep->valuep()) {
+                entryp->drivenWhole();
+            }
         }
         // Discover variables used in bit definitions, etc
         iterateChildren(nodep);
@@ -330,16 +334,18 @@ private:
         // Any variable
         if (nodep->lvalue()
             && !VN_IS(nodep, VarXRef)) {  // Ignore interface variables and similar ugly items
-            if (m_inProcAssign && !nodep->varp()->varType().isProcAssignable()) {
+            if (m_inProcAssign && !nodep->varp()->varType().isProcAssignable()
+                && !nodep->varp()->isDeclTyped()
+                && !nodep->varp()->isFuncLocal()) {
                 nodep->v3warn(PROCASSWIRE, "Procedural assignment to wire, perhaps intended var"
                               " (IEEE 2017 6.5): "
-                              +nodep->prettyName());
+                              +nodep->prettyNameQ());
             }
             if (m_inContAssign && !nodep->varp()->varType().isContAssignable()
                 && !nodep->fileline()->language().systemVerilog()) {
                 nodep->v3warn(CONTASSREG, "Continuous assignment to reg, perhaps intended wire"
                               " (IEEE 2005 6.1; Verilog only, legal in SV): "
-                              +nodep->prettyName());
+                              +nodep->prettyNameQ());
             }
         }
         for (int usr=1; usr<(m_alwaysCombp?3:2); ++usr) {
@@ -369,7 +375,6 @@ private:
         {
             m_inProcAssign = true;
             iterateChildren(nodep);
-            m_inProcAssign = false;
         }
         m_inProcAssign = prevProc;
     }
@@ -378,18 +383,16 @@ private:
         {
             m_inProcAssign = true;
             iterateChildren(nodep);
-            m_inProcAssign = false;
         }
         m_inProcAssign = prevProc;
     }
     virtual void visit(AstAssignW* nodep) {
-        bool prevCont = m_inProcAssign;
+        bool prevCont = m_inContAssign;
         {
             m_inContAssign = true;
             iterateChildren(nodep);
-            m_inContAssign = false;
         }
-        m_inProcAssign = prevCont;
+        m_inContAssign = prevCont;
     }
     virtual void visit(AstAlways* nodep) {
         AstAlways* prevAlwp = m_alwaysCombp;
@@ -427,7 +430,7 @@ private:
         iterateChildren(nodep);
     }
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     explicit UndrivenVisitor(AstNetlist* nodep) {
         m_inBBox = false;
         m_inContAssign = false;
