@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: Verilator: Add temporaries, such as for unroll nodes
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -27,7 +27,7 @@
 //      Convert remaining FORs to WHILEs
 //
 //*************************************************************************
-
+
 #include "config_build.h"
 #include "verilatedos.h"
 
@@ -56,8 +56,8 @@ private:
     bool                m_varAssignHit;         // Assign var hit
     bool                m_generate;             // Expand single generate For loop
     string              m_beginName;            // What name to give begin iterations
-    V3Double0           m_statLoops;            // Statistic tracking
-    V3Double0           m_statIters;            // Statistic tracking
+    VDouble0            m_statLoops;            // Statistic tracking
+    VDouble0            m_statIters;            // Statistic tracking
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -107,21 +107,22 @@ private:
         // Initial value check
         AstAssign* initAssp = VN_CAST(initp, Assign);
         if (!initAssp) return cantUnroll(nodep, "no initial assignment");
-        if (initp->nextp() && initp->nextp()!=nodep) nodep->v3fatalSrc("initial assignment shouldn't be a list");
+        UASSERT_OBJ(!(initp->nextp() && initp->nextp()!=nodep), nodep,
+                    "initial assignment shouldn't be a list");
         if (!VN_IS(initAssp->lhsp(), VarRef)) return cantUnroll(nodep, "no initial assignment to simple variable");
         //
         // Condition check
-        if (condp->nextp()) nodep->v3fatalSrc("conditional shouldn't be a list");
+        UASSERT_OBJ(!condp->nextp(), nodep, "conditional shouldn't be a list");
         //
         // Assignment of next value check
         AstAssign* incAssp = VN_CAST(incp, Assign);
         if (!incAssp) return cantUnroll(nodep, "no increment assignment");
-        if (incAssp->nextp()) nodep->v3fatalSrc("increment shouldn't be a list");
+        UASSERT_OBJ(!incAssp->nextp(), nodep, "increment shouldn't be a list");
 
         m_forVarp = VN_CAST(initAssp->lhsp(), VarRef)->varp();
         m_forVscp = VN_CAST(initAssp->lhsp(), VarRef)->varScopep();
         if (VN_IS(nodep, GenFor) && !m_forVarp->isGenVar()) {
-            nodep->v3error("Non-genvar used in generate for: "<<m_forVarp->prettyName()<<endl);
+            nodep->v3error("Non-genvar used in generate for: "<<m_forVarp->prettyNameQ()<<endl);
         }
         if (m_generate) V3Const::constifyParamsEdit(initAssp->rhsp());  // rhsp may change
 
@@ -190,10 +191,7 @@ private:
     bool simulateTree(AstNode *nodep, const V3Number *loopValue,
                       AstNode *dtypep, V3Number &outNum) {
         AstNode* clonep = nodep->cloneTree(true);
-        if (!clonep) {
-            nodep->v3fatalSrc("Failed to clone tree");
-            return false;
-        }
+        UASSERT_OBJ(clonep, nodep, "Failed to clone tree");
         if (loopValue) {
             m_varValuep = new AstConst(nodep->fileline(), *loopValue);
             // Iteration requires a back, so put under temporary node
@@ -279,7 +277,7 @@ private:
         if (initp) {
             initp->unlinkFrBack();  // Always a single statement; nextp() may be nodep
             // Don't add to list, we do it once, and setting loop index isn't
-            // needed as we're constant propagating it
+            // needed if we have > 1 loop, as we're constant propagating it
         }
         if (precondsp) {
             precondsp->unlinkFrBackWithNext();
@@ -354,6 +352,10 @@ private:
                 }
             }
         }
+        if (!newbodysp) {  // initp might have effects after the loop
+            newbodysp = initp;  // Maybe NULL
+            initp = NULL;
+        }
         // Replace the FOR()
         if (newbodysp) nodep->replaceWith(newbodysp);
         else nodep->unlinkFrBack();
@@ -460,13 +462,13 @@ private:
     }
 
 public:
-    // CONSTUCTORS
+    // CONSTRUCTORS
     UnrollVisitor() { init(false, ""); }
     virtual ~UnrollVisitor() {
         V3Stats::addStatSum("Optimizations, Unrolled Loops", m_statLoops);
         V3Stats::addStatSum("Optimizations, Unrolled Iterations", m_statIters);
     }
-    // METHORS
+    // METHODS
     void init(bool generate, const string& beginName) {
         m_forVarp = NULL;
         m_forVscp = NULL;
