@@ -2,7 +2,7 @@
 //*************************************************************************
 // DESCRIPTION: verilator_coverage: main()
 //
-// Code available from: http://www.veripool.org/verilator
+// Code available from: https://verilator.org
 //
 //*************************************************************************
 //
@@ -17,7 +17,7 @@
 // GNU General Public License for more details.
 //
 //*************************************************************************
-
+
 // Cheat for speed and compile .cpp files into one object
 #include "config_build.h"
 #ifndef HAVE_CONFIG_BUILD
@@ -59,9 +59,9 @@ bool VlcOptions::onoff(const char* sw, const char* arg, bool& flag) {
     // if sw=="-noarg", then return true (found it), and flag=false
     // else return false
     if (arg[0]!='-') v3fatalSrc("OnOff switches must have leading dash.");
-    if (0==strcmp(sw,arg)) { flag = true; return true; }
-    else if (0==strncmp(sw,"-no",3) && (0==strcmp(sw+3,arg+1))) { flag = false; return true; }
-    else if (0==strncmp(sw,"-no-",4) && (0==strcmp(sw+4,arg+1))) { flag = false; return true; }
+    if (0==strcmp(sw, arg)) { flag = true; return true; }
+    else if (0==strncmp(sw, "-no", 3) && (0==strcmp(sw+3, arg+1))) { flag = false; return true; }
+    else if (0==strncmp(sw, "-no-", 4) && (0==strcmp(sw+4, arg+1))) { flag = false; return true; }
     return false;
 }
 
@@ -73,40 +73,40 @@ void VlcOptions::parseOptsList(int argc, char** argv) {
     for (int i=0; i<argc; )  {
         UINFO(9, " Option: "<<argv[i]<<endl);
         if (argv[i][0]=='-') {
-            const char *sw = argv[i];
+            const char* sw = argv[i];
             bool flag = true;
             // Allow gnu -- switches
             if (sw[0]=='-' && sw[1]=='-') ++sw;
             if (0) {}
             // Single switches
-            else if ( onoff   (sw, "-annotate-all", flag/*ref*/) ) { m_annotateAll = flag; }
-            else if ( onoff   (sw, "-rank", flag/*ref*/) ) { m_rank = flag; }
-            else if ( onoff   (sw, "-unlink", flag/*ref*/) )    { m_unlink = flag; }
+            else if (onoff  (sw, "-annotate-all", flag/*ref*/) ) { m_annotateAll = flag; }
+            else if (onoff  (sw, "-rank", flag/*ref*/) ) { m_rank = flag; }
+            else if (onoff  (sw, "-unlink", flag/*ref*/) )    { m_unlink = flag; }
             // Parameterized switches
-            else if ( !strcmp (sw, "-annotate-min") && (i+1)<argc ) {
+            else if (!strcmp(sw, "-annotate-min") && (i+1)<argc ) {
                 shift;
                 m_annotateMin = atoi(argv[i]);
             }
-            else if ( !strcmp (sw, "-annotate") && (i+1)<argc ) {
+            else if (!strcmp(sw, "-annotate") && (i+1)<argc ) {
                 shift;
                 m_annotateOut = argv[i];
             }
-            else if ( !strcmp (sw, "-debug") ) {
+            else if (!strcmp(sw, "-debug") ) {
                 V3Error::debugDefault(3);
             }
-            else if ( !strcmp (sw, "-debugi") && (i+1)<argc ) {
+            else if (!strcmp(sw, "-debugi") && (i+1)<argc ) {
                 shift;
                 V3Error::debugDefault(atoi(argv[i]));
             }
-            else if ( !strcmp (sw, "-V") ) {
+            else if (!strcmp(sw, "-V") ) {
                 showVersion(true);
                 exit(0);
             }
-            else if ( !strcmp (sw, "-version") ) {
+            else if (!strcmp(sw, "-version") ) {
                 showVersion(false);
                 exit(0);
             }
-            else if ( !strcmp (sw, "-write") && (i+1)<argc ) {
+            else if (!strcmp(sw, "-write") && (i+1)<argc ) {
                 shift;
                 m_writeFile = argv[i];
             }
@@ -139,7 +139,7 @@ void VlcOptions::showVersion(bool verbose) {
     cout << "License Version 2.0.\n";
 
     cout <<endl;
-    cout << "See http://www.veripool.org/verilator for documentation\n";
+    cout << "See https://verilator.org for documentation\n";
 }
 
 //######################################################################
